using System;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using FitConnect;
using FitConnect.Encryption;
using FluentAssertions;
using IdentityModel.Jwk;
using Microsoft.Extensions.Logging;
using NUnit.Framework;

namespace SenderTest;

public class RootCertificatesTests {
    private ILogger _logger;

    [OneTimeSetUp]
    public void OneTimeSetup() {
        _logger = LoggerFactory.Create(
            b => {
                b.AddSimpleConsole();
                b.SetMinimumLevel(LogLevel.Information);
            }).CreateLogger("EncryptionTests");
    }

    [Test]
    public void LoadRootCertificates() {
        var certificates = CertificateHelper.GetRootCertificates().Result;
        certificates.Count.Should().BeGreaterThan(2);
    }

    [Test]
    public void ProdSubmission_ValidCertificate() {
        var certificate = File.ReadAllText("./Certificates/ProdZustelldienst.json");
        var key = new Microsoft.IdentityModel.Tokens.JsonWebKeySet(certificate).Keys.First();
        var result = new CertificateHelper(_logger).ValidateCertificate(key, LogLevel.Trace)
            .Result;

        result.Should().BeTrue();
    }

    [Test]
    public void TestPki_ValidCertificate() {
        var certificate =
            X509Certificate2.CreateFromPem(
                File.ReadAllText("./Certificates/ValidCertificateFromTestPki.pem"));
        
        var root =
            X509Certificate2.CreateFromPem(
                File.ReadAllText("./Certificates/TestPkiRootCertificate.pem"));
        
        var result = new CertificateHelper(_logger).ValidateCertificate(certificate, new[] { root },
                null, LogLevel.Trace)
            .Result;

        result.Should().BeTrue();
    }
}
